@@ -12,7 +12,12 @@
 #include "psa/crypto.h"
 #include "psa_crypto_cipher.h"
 #include "psa_crypto_core.h"
+
+#if !defined(MBEDTLS_VERSION_MAJOR) || MBEDTLS_VERSION_MAJOR >= 4
+#include "mbedtls/private/cipher.h"
+#else
 #include "mbedtls/cipher.h"
+#endif
 
 #include "test/drivers/cipher.h"
 
